-- GLOBAL --
ATT = select(2, ...); 
-- MODULE --
local _G = getfenv(0);
setmetatable(ATT, {__index = _G});
setfenv(1, ATT);
-- LOCAL --
title		= "Alt-Tab Toggle"
version		= "1.0.1"
titleFull	= title.." version "..version

coreFrame = CreateFrame("Frame",nil,UIParent);
core	= LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0") -- 
local L = LibStub("AceLocale-3.0"):GetLocale(title, true)
local P; --db.profile

--~ ATTFrame = CreateFrame("Frame",nil,UIParent)
lastOnUpdate = 0;
ATTActive = false;
directory 	= "Interface\\Addons\\"..title.."\\";
timers = {};

--Globals
local CLEARED_AFK_							= _G.CLEARED_AFK;
local COMBATLOG_OBJECT_REACTION_HOSTILE_	= _G.COMBATLOG_OBJECT_REACTION_HOSTILE;
local COMBATLOG_OBJECT_TYPE_PLAYER_			= _G.COMBATLOG_OBJECT_TYPE_PLAYER;
local COMBAT_								= _G.COMBAT;
local ConsoleExec_							= _G.ConsoleExec;
local ENABLE_								= _G.ENABLE;
local EVENTS_LABEL_							= _G.EVENTS_LABEL;
local FACTION_INACTIVE_						= _G.FACTION_INACTIVE;
local GAMEOPTIONS_MENU_						= _G.GAMEOPTIONS_MENU;
local GetBattlefieldStatus_					= _G.GetBattlefieldStatus;
local GetCVar_								= _G.GetCVar;
local GetRealZoneText_						= _G.GetRealZoneText;
local GetTime_								= _G.GetTime;
local INVITE_								= _G.INVITE;
local InCombatLockdown_						= _G.InCombatLockdown;
local InterfaceOptionsFrame_OpenToCategory_	= _G.InterfaceOptionsFrame_OpenToCategory;
local IsInInstance_							= _G.IsInInstance;
local MARKED_AFK_							= _G.MARKED_AFK
local MAX_BATTLEFIELD_QUEUES_				= _G.MAX_BATTLEFIELD_QUEUES;
local PlaySoundFile_						= _G.PlaySoundFile;
local READY_CHECK_							= _G.READY_CHECK;
local SUMMON_								= _G.SUMMON;
local SetCVar_								= _G.SetCVar;
local UnitDebuff_							= _G.UnitDebuff;
local WHISPER_								= _G.WHISPER;
local bit_band								= _G.bit.band;
local pairs_								= _G.pairs;
local print_								= _G.print;
local select_								= _G.select;
local table_getn							= _G.table.getn;
local tostring_								= _G.tostring



-- Default Settings
defaultSettings = {profile = {}}
defaultSettings.profile.ATTBGGatesOpening	= true;
defaultSettings.profile.ATTBGPop			= true;
defaultSettings.profile.ATTCombat			= true;
defaultSettings.profile.ATTDeserter			= true;
defaultSettings.profile.ATTEnemyDetected	= false;
defaultSettings.profile.ATTFlight			= true;
defaultSettings.profile.ATTIdle				= true;
defaultSettings.profile.ATTInvite			= true;
defaultSettings.profile.ATTReadyCheck		= true;
defaultSettings.profile.ATTSummon			= true;
defaultSettings.profile.ATTWhisper			= true;
defaultSettings.profile.attAFKActivate		= false;
defaultSettings.profile.attAlertSound		= true;
defaultSettings.profile.attAltTabDetection	= true;
defaultSettings.profile.soundOnly			= false;


-- Echo Strings
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.


--Internal settings
local atDetectionWait	= 5;-- if our OnUpdate function hasn't updated in 5 seconds, assume we've alt tabbed.
ignoreEnemyZones = {
	[L["Shattrath City"]]	= true,
	[L["Dalaran"]] 			= true, --unitispvp
	[""]					= true,--Blank when we login, the GetZone API doesn't return the zone name.
}

local regEvents = {
	"CHAT_MSG_BG_SYSTEM_NEUTRAL",
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_WHISPER",
	"COMBAT_LOG_EVENT_UNFILTERED",
	"CONFIRM_SUMMON",
	"PARTY_INVITE_REQUEST",
	"PLAYER_CONTROL_GAINED",
	"PLAYER_ENTERING_WORLD",
	"PLAYER_REGEN_DISABLED",
	"READY_CHECK",
	"UNIT_AURA",
	"UPDATE_BATTLEFIELD_STATUS",
}

-- Ace3 Functions
function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("ATT_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")

	P = db.profile;

	core:RegisterChatCommand("att", "MySlashProcessorFunc")

	local config = LibStub("AceConfig-3.0");
	local dialog = LibStub("AceConfigDialog-3.0");
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, "ATT v"..version)
end



function core:OnEnable()
	coreFrame:SetScript("OnUpdate", function(this, duration)
		lastOnUpdate = GetTime_();
	end)

	for i, event in pairs_(regEvents) do 
		self:RegisterEvent(event);
	end
	
	if GetCVar_("Sound_EnableSoundWhenGameIsInBG") == 0 then
		print_(L["'Sounds in Background' is disabled, Enable it if you want sounds when alt tabbed."])
	end
	
	--make all options clickable.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = false;
		end
	end
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /att function brings up the UI options.	--
----------------------------------------------
	if core:IsEnabled() then
		if not input or input == "" then
			ATTToggleMode(nil, nil, "Slash command");
		else
			InterfaceOptionsFrame_OpenToCategory_(coreOpts)
		end
	else
		print_(L["ATT is disabled, enable it on the options screen first."])
	end
end



function core:OnDisable()
	coreFrame:SetScript("OnUpdate", nil)
	
	--make all the options unclickable except for the enable button.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = true;
		end
	end
	ATTToggleMode(false, false, L["Addon Disabled"])
end



----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

function IsAltTabbed()
	if InCombatLockdown_() then
		--we're in combat, probably not alt tabbed. note this doesn't return true when we enter combat.
		return false;
	end
	if GetCVar_("gxWindow") == 0 then
		if GetTime_() - lastOnUpdate > atDetectionWait then
			return true;
		end	
	end
	return false;
end


function core:CHAT_MSG_BG_SYSTEM_NEUTRAL(event, ...)
	local message  = ...-- ***
	message = message:lower();
	if	message:find(L["the battle for arathi basin has begun!"]) or --AB
		message:find(L["the battle for alterac valley has begun"]) or --AV
		message:find(L["let the battle for warsong gulch begin!"]) or --WSG
		message:find(L["the battle has begun"]) then --EotS

		if P.ATTBGGatesOpening == true then
			FocusWoW(L["Gates opened"]);
		end

	end
end

function core:CONFIRM_SUMMON(event, ...)
	if P.ATTSummon == true then
		FocusWoW(SUMMON_);
	end
end

function core:READY_CHECK(event, ...)
	if P.ATTReadyCheck == true then
		FocusWoW(L["Ready Check"]);
	end
end


function core:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	if ignoreEnemyZones[GetRealZoneText_()] then
		return;
	end

	if P.ATTEnemyDetected == true and not InBattleground() then
		local timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags  = select_(1, ...)-- ***

		if FlagIsPlayer(srcFlags) and FlagIsEnemy(srcFlags) then
			FocusWoW(L["Enemy detected (%s)"]:format(srcName));
		elseif FlagIsPlayer(dstFlags) and FlagIsEnemy(dstFlags) then
			FocusWoW(L["Enemy detected (%s)"]:format(dstName));
		end
	end
end

function FlagIsEnemy(flags)
	if bit_band(flags, COMBATLOG_OBJECT_REACTION_HOSTILE_) == COMBATLOG_OBJECT_REACTION_HOSTILE_ then
		return true;
	end
	return nil;
end

function FlagIsPlayer(flags)
	if bit_band(flags, COMBATLOG_OBJECT_TYPE_PLAYER_) == COMBATLOG_OBJECT_TYPE_PLAYER_ then
		return true;
	end
	return nil;
end

function core:PLAYER_ENTERING_WORLD(event, ...)
	lastFocusTime = GetTime_();--if we exit a BG, PLAYER_CONTROL_GAINED (flight land) will fire, we don't want that.
end

function core:PLAYER_REGEN_DISABLED(event, ...)
	if P.ATTCombat == true then
		FocusWoW(L["Entered combat"]);
	end
end

function core:CHAT_MSG_WHISPER(event, ...)
	if P.ATTWhisper == true then
		FocusWoW(L["Received whisper"]);
	end
end

function core:PARTY_INVITE_REQUEST(event, ...)
	if P.ATTInvite == true then
		FocusWoW(INVITE_);
	end
end

function core:PLAYER_CONTROL_GAINED(event, ...)
	if P.ATTFlight == true then
		FocusWoW(L["Flight land"]);
	end
end

function core:UPDATE_BATTLEFIELD_STATUS(event, ...)
	if P.ATTBGPop == true then
		local status, mapName, instanceID;
		for i=1, MAX_BATTLEFIELD_QUEUES_ do
			status, mapName, instanceID = GetBattlefieldStatus_(i)
			if status == "confirm" then
				FocusWoW(L["Battleground confirm"]);
			end
		end
	end
end


function core:CHAT_MSG_SYSTEM(event, ...)
	if P.attAFKActivate == true then
		local message, format  = ...;
		if message:find(MARKED_AFK_) then --You are now AFK: Away from Keyboard

			ATTToggleMode(true, nil, L["User went AFK"])
			return;
		elseif message:find(CLEARED_AFK_) then
			ATTToggleMode(false, nil, L["User no longer AFK"])
			return;
		end
	end
end


isDeserter = false;
function core:UNIT_AURA(event, ...)
	if P.ATTDeserter == true then
		local unitID  = ...
		if unitID == "player" then
			local buffOn = IsDebuffOn(L["Deserter"], unitID);
			if isDeserter == false then
				if buffOn then
					isDeserter = true;
				end
			else
				if not buffOn then
					FocusWoW(L["Deserter expired"]);
					isDeserter = false;
				end
			end
		end
	end
	if P.ATTIdle == true then
		local unitID  = ...
		if unitID == "player" then
			if IsDebuffOn(L["Idle"], unitID) then
				FocusWoW(L["Idle debuff gained"]);
			elseif IsDebuffOn(FACTION_INACTIVE_, unitID) then
				FocusWoW(L["Inactive debuff gained"]);
			end
		end
	end
end

------------------------------------------
function IsDebuffOn(spellName, unitID)	--
-- Check if a spell exists on a unitID.	--
------------------------------------------
	local name;
	for i = 1, 40 do
		name  =  UnitDebuff_(unitID, i)
		if not name then break; end
		if name == spellName then
			return true;
		end
	end
	return false;
end

------------------------------
function Debug(from, ...)	--	/script DA.Debug("hey","bye")
-- simple print function.	--
------------------------------
	local tbl  = {...}
	local msg = tostring_(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring_(tbl[i]);
	end
	print_(strDebugFrom:format(from).." "..msg)
end

--------------------------------------------------
function FocusWoW(reason)						--
-- Check if ATT is active, if so toggle WoW.	--
--------------------------------------------------
	if (P.attAltTabDetection == true and IsAltTabbed()) or ATTActive == true then --
		HardFocusWoW(reason, true)
		ATTActive = false;
	end
end


lastFocusTime = 0;
--------------------------------------------------------------
function HardFocusWoW(reason, force)						--	/script ATT.HardFocusWoW()
-- Toggle WoW if we haven't toggled in the last 5 seconds.	--
--------------------------------------------------------------
	if GetTime_() - lastFocusTime < 5 then
		return;
	end
	if ATTActive == true or force == true then
		--I forget why this windowmode code is here, /shrug
--~ 		local WindowMode = GetCVar_("gxWindow");	--1 = windowmode, 0 = fullscreen.
--~ 		if (WindowMode == 1) then
--~ 			SetCVar_("gxWindow", "1"); --window mode
--~ 		else
--~ 			SetCVar_("gxWindow", "0"); --window full
--~ 		end
			
		if P.attAlertSound == true then -- and P.ATTDelayFocus == false
			TurnSoundsOn()
			PlaySoundFile_(directory.."Sounds\\ATTalert.mp3")
			core:ScheduleTimer("TurnSoundsOff", 2)--turn sounds off in 2 seconds.
		end
		
		if P.soundOnly == false then
			ConsoleExec_("gxRestart"); -- /console gxRestart
		end
		
		lastFocusTime = GetTime_();
		if reason then
			ATTToggleMode(false, true)
			print_(L["Alt-Tab Toggle: |cffffff00%s|r"]:format(reason))
		else
			ATTToggleMode(false)
		end

	end
end



local soundSettings = {};
function TurnSoundsOn()
	--Sound_EnableSoundWhenGameIsInBG
	soundSettings.ALL = GetCVar_("Sound_EnableAllSound")
	soundSettings.ALLVol = GetCVar_("Sound_MasterVolume")
	soundSettings.SFX = GetCVar_("Sound_EnableSFX")
	soundSettings.SFXVol = GetCVar_("Sound_SFXVolume")
	soundSettings.BG = GetCVar_("Sound_EnableSoundWhenGameIsInBG")

	SetCVar_("Sound_EnableAllSound", 1)
	SetCVar_("Sound_MasterVolume", 1)
	SetCVar_("Sound_EnableSFX", 1)
	SetCVar_("Sound_SFXVolume", 1)
	SetCVar_("Sound_EnableSoundWhenGameIsInBG", 1)--this can't be set while alt tabbed? WTF!
end

function core:TurnSoundsOff()
	if soundSettings.ALL then
		SetCVar_("Sound_EnableAllSound", soundSettings.ALL)
	end
	if soundSettings.ALLVol then
		SetCVar_("Sound_MasterVolume", soundSettings.ALLVol)
	end
	if soundSettings.SFX then
		SetCVar_("Sound_EnableSFX", soundSettings.SFX)
	end
	if soundSettings.SFXVol then
		SetCVar_("Sound_SFXVolume", soundSettings.SFXVol)
	end
	if soundSettings.BG then
		SetCVar_("Sound_EnableSoundWhenGameIsInBG", soundSettings.BG)
	end
end

function ATTToggleMode(mode, silent, reason)
	if mode == nil then mode = not ATTActive; end
	if ATTActive ~= mode then
		ATTActive = mode;
		if (ATTActive == true) then
			if not silent then
				print_(L["Alt-Tab Toggle: |cff00ff00Activated|r"], "||", reason)
			end
		else
			if not silent then
				print_(L["Alt-Tab Toggle: |cffff0000Deactivated|r"], "||", reason)
			end
		end
	end
end

function InBattleground()
	local iActive, iType = IsInInstance_()
	if iActive and iType == "pvp" then
		--We're in a battlefield.
		return true;
	end
	return false;
end



----------------
--     UI     --
----------------
CoreOptionsTable = {
	name = titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = GAMEOPTIONS_MENU_,
			type = "group",
			order = 1,
			args={}
		},
	},
}

CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= ENABLE_,
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable();
		else
			core:Disable();
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.attAFKActivate = {
	name = L["AFK Activate"],
	desc = L["When you go away from the keyboard (/afk), ATT will activate then you're no longer afk."],
	type = "toggle",
	order	= 2,
	set = function(info,val) 
		P.attAFKActivate = val;
	end,
	get = function(info) return P.attAFKActivate end
}

CoreOptionsTable.args.core.args.attAlertSound = {
	type = "toggle",	order	= 3,
	name = L["Alert Sound"],
	desc = L["Play a sound when toggling"],
	set = function(info,val) 
		P.attAlertSound = val;
	end,
	get = function(info) return P.attAlertSound end
}

CoreOptionsTable.args.core.args.soundOnly = {
	name = L["Sound Only"],
	desc = L["Only play sound on toggle event, don't force WoW to foreground."],
	type = "toggle",
	order	= 4,
	set = function(info,val) 
		P.soundOnly = val;
	end,
	get = function(info) return P.soundOnly end
}


CoreOptionsTable.args.core.args.attAltTabDetection = {
	name = L["Alt Tab Detection"],
	desc = L["If your FPS hasen't refreshed in 5 seconds we assume you've alt tabbed."],
	type = "toggle",
	order	= 5,
	set = function(info,val) 
		P.attAltTabDetection = val;
	end,
	get = function(info) return P.attAltTabDetection end
}


---------
CoreOptionsTable.args.core.args.eventHeader = {
	type = "header", order	= 10,
	width	= "full",
	name	= EVENTS_LABEL_,
}

CoreOptionsTable.args.core.args.ATTIdle = {
	type = "toggle",	order	= 11,
	name = L["BG Idle"],
	desc = L["Gain battleground idle debuff"],
	set = function(info,val) 
		P.ATTIdle = val;
	end,
	get = function(info) return P.ATTIdle end
}
CoreOptionsTable.args.core.args.ATTDeserter = {
	type = "toggle",	order	= 12,
	name = L["Deserter Expire"],
	desc = L["Deserter debuff expires"],
	set = function(info,val) 
		P.ATTDeserter = val;
	end,
	get = function(info) return P.ATTDeserter end
}

CoreOptionsTable.args.core.args.ATTWhisper = {
	type = "toggle",	order	= 13,
	name = WHISPER_,
	desc = L["Receive whisper"],
	set = function(info,val) 
		P.ATTWhisper = val;
	end,
	get = function(info) return P.ATTWhisper end
}

CoreOptionsTable.args.core.args.ATTCombat = {
	type = "toggle",	order	= 14,
	name = COMBAT,
	desc = L["Enter combat"],
	set = function(info,val) 
		P.ATTCombat = val;
	end,
	get = function(info) return P.ATTCombat end
}

CoreOptionsTable.args.core.args.ATTFlight = {
	type = "toggle",	order	= 15,
	name = L["Flight"],
	desc = L["Flight land"],
	set = function(info,val) 
		P.ATTFlight = val;
	end,
	get = function(info) return P.ATTFlight end
}

CoreOptionsTable.args.core.args.ATTReadyCheck = {
	type = "toggle",	order	= 16,
	name = READY_CHECK_,
	desc = L["Group Ready Check"],
	set = function(info,val) 
		P.ATTReadyCheck = val;
	end,
	get = function(info) return P.ATTReadyCheck end
}
CoreOptionsTable.args.core.args.ATTSummon = {
	type = "toggle",	order	= 17,
	name = SUMMON_,
	desc = L["Summon requests"],
	set = function(info,val) 
		P.ATTSummon = val;
	end,
	get = function(info) return P.ATTSummon end
}
CoreOptionsTable.args.core.args.ATTInvite = {
	type = "toggle",	order	= 18,
	name = INVITE_,
	desc = L["Group invite"],
	set = function(info,val) 
		P.ATTInvite = val;
	end,
	get = function(info) return P.ATTInvite end
}
CoreOptionsTable.args.core.args.ATTBGPop = {
	type = "toggle",	order	= 19,
	name = L["BG confirm"],
	desc = L["Battleground confirm to enter"],
	set = function(info,val) 
		P.ATTBGPop = val;
	end,
	get = function(info) return P.ATTBGPop end
}
CoreOptionsTable.args.core.args.ATTEnemyDetected = {
	type = "toggle",	order	= 20,
	name = L["Enemy Detected"],
	desc = L["When a Enemy is detected in combatlog. (non-BG/city only)"],
	set = function(info,val) 
		P.ATTEnemyDetected = val;
	end,
	get = function(info) return P.ATTEnemyDetected end
}
CoreOptionsTable.args.core.args.ATTBGGatesOpening = {
	type = "toggle",	order	= 21,
	name = L["Gates Opening"],
	desc = L["BG Gates opening"],
	set = function(info,val) 
		P.ATTBGGatesOpening = val;
	end,
	get = function(info) return P.ATTBGGatesOpening end
}